[−][src]Trait logos::Extras
Helper trait that can be injected into the Lexer to handle things that
aren't necessarily tokens, such as comments or Automatic Semicolon Insertion
in JavaScript.
Provided Methods
fn on_advance(&mut self)
Method called by the Lexer when a new token is about to be produced.
fn on_whitespace(&mut self, _byte: u8)
Method called by the Lexer when a white space byte has been encountered.
Implementations on Foreign Types
impl Extras for ()[src]
impl Extras for ()Default Extras with no logic
fn on_advance(&mut self)[src]
fn on_advance(&mut self)fn on_whitespace(&mut self, _byte: u8)[src]
fn on_whitespace(&mut self, _byte: u8)